@@ -3,7 +3,7 @@
 
 package HiD::App::Command::server;
 {
-  $HiD::App::Command::server::VERSION = '0.7';
+  $HiD::App::Command::server::VERSION = '0.8';
 }
 BEGIN {
   $HiD::App::Command::server::AUTHORITY = 'cpan:GENEHACK';
@@ -23,6 +23,7 @@ use feature     qw/ unicode_strings /;
 use namespace::autoclean;
 
 use Plack::Runner;
+use AnyEvent::Filesys::Notify;
 
 
 has port => (
@@ -60,8 +61,6 @@ sub _run {
 
   my $app = HiD::Server->new( root => $self->destination )->to_app;
 
-  my %args = ( -p => $self->port );
-
   # auto refresh
   if ( $self->auto_refresh ) {
       my @dirs;
@@ -76,18 +75,33 @@ sub _run {
           push @dirs, map { $_->input_filename } @{ $self->hid->$dir };
       }
 
-      $args{'-R'} = join ',', @dirs;
-      $args{'-r'} = 1;
-      my $_app = $app;
-      $app = \sub {
-          say 'Rebuild ... ';
-          $self->publish;
-          $_app;
-      };
+      say "*** auto refresh, watching at:";
+      say foreach @dirs;
+      say "***";
+      my $building = 0;
+      AnyEvent::Filesys::Notify->new(
+          dirs => \@dirs,
+          interval => 1.0,
+          filter => sub { shift !~ /\.(swap|#|~)$/ },
+          backend => $^O eq 'darwin' ? 'KQueue' : '', # Mac::FSEvents do not support watch at file
+          cb => sub {
+              return if $building;
+              $building = 1;
+              say 'Rebuilding ... ';
+              $self->_set_hid( HiD->new({
+                  cli_opts => $self->hid->cli_opts,
+                  config_file => $self->hid->config_file,
+              }));
+              $self->publish;
+              $building = 0;
+          }
+      );
+      eval "use Twiggy; 1" or
+          die "You should install Twiggy when use --auto_refresh option.";
   }
 
   my $runner = Plack::Runner->new;
-  $runner->parse_options(%args);
+  $runner->parse_options( -p => $self->port );
   $runner->run($app);
 }
 
@@ -154,7 +168,7 @@ sub commands.
 
 =head1 VERSION
 
-version 0.7
+version 0.8
 
 =head1 AUTHOR
 
